#include <algorithm>

#include <ardour/region.h>

#include "regionview.h"
#include "region_selection.h"

using namespace ARDOUR;
using namespace SigC;

struct RegionComparator {
	bool operator() (const RegionView* a, const RegionView* b) const {
		if (a->region.layer() == b->region.layer()) {
			return false;
		} else {
			return a->region.layer() < b->region.layer();
		}
	}
};

RegionSelection::RegionSelection ()
{
	_current_start = 0;
	_current_end = 0;
}

RegionSelection::RegionSelection (const RegionSelection& other)
{
	RegionComparator cmp;

	for (list<RegionView*>::const_iterator i = other.begin(); i != other.end(); ++i) {
		add (*i, false);
	}

	sort (cmp);
	_current_start = other._current_start;
	_current_end = other._current_end;
}

RegionSelection&
RegionSelection::operator= (const RegionSelection& other)
{
	RegionComparator cmp;

	if (this != &other) {

		for (list<RegionView*>::const_iterator i = other.begin(); i != other.end(); ++i) {
			add (*i, false);
		}

		sort (cmp);

		_current_start = other._current_start;
		_current_end = other._current_end;
	}

	return *this;
}

void
RegionSelection::add (RegionView* rv, bool dosort)
{
	RegionComparator cmp;

	rv->GoingAway.connect (slot (*this, &RegionSelection::remove_it));

	push_back (rv);

	if (dosort) {
		sort(cmp);
	}

	if (rv->region.first_frame() < _current_start || empty()) {
		_current_start = rv->region.first_frame();
	}
	
	if (rv->region.last_frame() > _current_end || empty()) {
		_current_end = rv->region.last_frame();
	}
}

void
RegionSelection::remove_it (TimeAxisViewItem *item)
{
	RegionView* rv = dynamic_cast<RegionView*> (item);

	if (rv) {
		remove (rv);
	}
}

bool
RegionSelection::remove (RegionView* rv)
{
	list<RegionView*>::iterator i;

	if ((i = find (begin(), end(), rv)) != end()) {

		erase (i);

		if (empty()) {

			_current_start = 0;
			_current_end = 0;

		} else {

			Region& region ((*i)->region);

			if (region.first_frame() == _current_start) {
				
				/* reset current start */
				
				jack_nframes_t ref = max_frames;
				
				for (i = begin (); i != end(); ++i) {
					if (region.first_frame() < ref) {
						ref = region.first_frame();
					}
				}
				
				_current_start = ref;
				
			}
			
			if (region.last_frame() == _current_end) {

				/* reset current end */
				
				jack_nframes_t ref = 0;
				
				for (i = begin (); i != end(); ++i) {
					if (region.first_frame() > ref) {
						ref = region.first_frame();
					}
				}
				
				_current_end = ref;
			}
		}

		return true;
	}

	return false;
}
